#!/usr/bin/python
# -*- coding: utf-8 -*-
##      google.cgi
##
##
## Copyright (C) 2010 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import cgitb
cgitb.enable()
import GeoIP
import cgi
import gettext
import locale
import os
import random
import sys

_  = gettext.gettext

def main(argv) :
	setup_locale()
	print_html(get_markers_js(get_markers_dicts()))

def get_markers_dicts() :
	"""
	This function gets and converts the cgi variables 'marker' to a list of
	dicts.
	"""
	# http://www.maxmind.com/app/geolitecity
	geo_city_db = "../GeoLiteCity.dat"

	form = cgi.FieldStorage()
	markers = form.getvalue("marker")

	if markers is None :
		# no markers
		markers_ = []
	elif not isinstance(markers, list) :
		# only one marker
		markers = [markers]

	markers_dicts = []

	gi = GeoIP.open(geo_city_db, GeoIP.GEOIP_STANDARD)

	for marker in markers :
		m = process_marker_dict(gi, marker)
		if m is not None:
			markers_dicts.append(m)

	return markers_dicts

def process_marker_dict(gi, marker) :
	"""
	This function converts the cgi variable 'marker' to a dict.
	"""
	(ip, hostname, thop) = marker.split("==")
	thop = float(thop)

	gir = gi.record_by_addr(ip)
	if gir is None : return None

	lat = gir['latitude']
	lon = gir['longitude']
	city = gir['city']

	desc = "<h3>%s</h3>" % hostname
	desc += _("IP: %s <br />") % ip
	desc += _("Country: %s <br />") % gir['country_name']
	desc += _("City: %s <br />") % gir['city']
	if thop > 0 :
		desc += _("Latency: %.2f <br />") % thop

	return {
		"lat": lat,
		"lon": lon,
		"title": hostname,
		"desc": desc,
	}

def get_markers_js(markers) :
	"""
	This function converts list of markers to javascript code.
	"""
	markers_str = ""
	for i in xrange(len(markers)) :
		k = find_marker(markers[i], markers[(i+1):])
		if k >= 0 :
			k += i + 1
			markers[k]["title"] += ", " + markers[i]["title"]
			markers[k]["desc"] += "<hr />" + markers[i]["desc"]
		else :
			markers_str += process_marker_js(markers[i])

	return markers_str

def process_marker_js(marker) :
	"""
	This function converts the marker to javascript code.
	"""
	return '''{
		"lat": %f,
		"lon": %f,
		"title": "%s",
		"desc": "%s",
	},''' % (marker["lat"], marker["lon"], marker["title"], marker["desc"])

def find_marker(marker, markers) :
	for i in xrange(len(markers)) :
		if marker["lat"] == markers[i]["lat"] and\
			marker["lon"] == markers[i]["lon"] :
			return i

	return -1

def setup_locale() :
	"""
	This function tunes the locale
	"""
	try :
		l = os.environ["HTTP_ACCEPT_LANGUAGE"].split(";")[0].split(",")[0]
		os.environ["LC_ALL"] = "%s.UTF-8" % l
	except :
		pass

	if locale.getdefaultlocale() != (None, None) :
		gettext.bindtextdomain("gnetworktester", "../locale")
		gettext.textdomain("gnetworktester")

def print_html (markers) :
	"""
	This function prints results
	"""

	print 'Content-Type: text/html; charset=utf-8' # HTML is following
	print										  # blank line, end of headers

	print \
'''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
	<title>gnetworktester map</title>

	<meta name="viewport" content="initial-scale=1.0, user-scalable=no" />
	<meta http-equiv="Content-type" content="text/html; charset=UTF-8" />

	<!--<script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false&amp;key=ABQIAAAAdSuc0zjkgYMQfR9GWdwEiBRn5K_w75_fKzP9dBS2QMwuGvoqmxR65ckpZwSiU4Poit0k3X8gP8JB1g"></script>-->
	<script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false"></script>
	<script type="text/javascript" src="/maps.js"></script>
	<script type="text/javascript">

		var map_width;
		var map_height;

		function initialize ()
		{
			map_width = window.innerWidth - 20;
			map_height = window.innerHeight - 20;

			var mapdiv = document.getElementById('map');
			mapdiv.style.width = map_width + 'px';
			mapdiv.style.height = map_height + 'px';

			var map = google_map("map",
				[%s]
			);
		}
	</script>
</head>
<body onload="initialize()">
  <div id="map"></div>
</body>
</html>
''' % (markers)

if __name__ == "__main__":
	main(sys.argv)

