/*
 *      main.js
 *
 *
 * Copyright (C) 2010 Max V. Stotsky <ms@pereslavl.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; version 3 or later.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

var Re = 6378100; // Equatorial radius of Earth
var Rp = 6356800; // Polar radius of Earth

var google_zooms = [ // meters in one pixel
		0.16, 0.31, 0.63, 1.25, 2.50, 5.00, 10.00, 20.00, 40.00, 80.00, 160.00,
		307.69, 625.00, 1250.00, 2500.00, 5263.16, 10526.32, 21052.63,
		41666.67, 83333.33,
];

function google_map(id, markers)
	/*
	 * This function creates a google map and puts markers on it.
	 *
	 * 'id' is id of an html tag for the map.
	 * 'markers' is a list of markers.
	 *  'markers[i]' is a hash. List of keys of the hash:
	 *  'lat', 'lon' is coordinates of the marker.
	 *  'title' is short hint on mouseOver.
	 *  'desc' is a long description for the maker.
	 */
{
	// to create the map
	var myOptions = {
		mapTypeId: google.maps.MapTypeId.ROADMAP
	};

	var map = new google.maps.Map(document.getElementById(id), myOptions);

	// to process markers
	// 'max_lat' and 'max_lon' are the highest among latitude and longitude markers.
	// 'min_lat' and 'min_lon' are the lowest among latitude and longitude markers.
	var max_lat = -90;
	var max_lon = -180;
	var min_lat = 90;
	var min_lon = 180;
	for (var m in markers) {
		google_create_marker(map, markers[m]);

		max_lat = Math.max(markers[m]['lat'], max_lat);
		max_lon = Math.max(markers[m]['lon'], max_lon);
		min_lat = Math.min(markers[m]['lat'], min_lat);
		min_lon = Math.min(markers[m]['lon'], min_lon);
	}

	// margins
	var fw = Math.max( (max_lat - min_lat)*0.2, (max_lon - min_lon)*0.2 );
	max_lat += fw;
	max_lon += fw;
	min_lat -= fw;
	min_lon -= fw;

	// to calculate center
	var clat = (max_lat + min_lat) / 2;
	var clon = (max_lon + min_lon) / 2;

	// 'lp' is number of meters in one degree of the latitude
	var lp = deg2rad(1) * Rp;
	// 'le' is number of meters in one degree of the longitude
	var le = deg2rad(1) * Re * Math.cos(deg2rad(clon));

	// trying to find optimal zoom
	var zoom = google_zooms.length - 1;
	for (var i in google_zooms) {
		var pxp = (google_zooms[i] / lp) * map_height;
		var pxe = (google_zooms[i] / le) * map_width;
		if (clat - pxp < min_lat && max_lat < clat + pxp &&
			clon - pxe < min_lon && max_lon < clon + pxe) {
			// if all markers is visible
			break;
		}
		zoom--;
	}

	// setting up the map
	map.setCenter(new google.maps.LatLng(clat, clon));
	map.setZoom(zoom);

	return map;
}

function google_create_marker(map, marker)
	/*
	 * This function creates marker on a google map
	 *
	 * 'map' is the google map object.
	 * 'marker' is a hash. List of keys of the hash:
	 *  'lat', 'lon' are the coordinates of the marker.
	 *  'title' is short hint on mouseOver.
	 *  'desc' is a long description for the maker.
	 */
{
	var mrk = new google.maps.Marker({
		position: new google.maps.LatLng(marker['lat'], marker['lon']),
		map: map,
		title: marker['title'],
	});

	var infowindow = new google.maps.InfoWindow({
		content: marker['desc'],
	});

	google.maps.event.addListener(mrk, 'click', function() {
		infowindow.open(map, mrk);
	});
}

function deg2rad(deg)
	/*
	 * This function converts degrees to radians
	 */
{
	return deg * Math.PI / 180;
}

